/*
 * heremaps-uc
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Jongmun Woo <jongmun.woo@samsung.com>, Seechan Kim <cbible.kim@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include "heremaps-uc-common.h"
#include <stdlib.h>
#include <system_info.h>

typedef enum {
	TIZEN_PROFILE_UNKNOWN = 0,
	TIZEN_PROFILE_MOBILE = 0x1,
	TIZEN_PROFILE_WEARABLE = 0x2,
	TIZEN_PROFILE_TV = 0x4,
	TIZEN_PROFILE_IVI = 0x8,
	TIZEN_PROFILE_COMMON = 0x10,
} tizen_profile_t;

static tizen_profile_t _get_tizen_profile()
{
	static tizen_profile_t profile = TIZEN_PROFILE_UNKNOWN;
	if (__builtin_expect(profile != TIZEN_PROFILE_UNKNOWN, 1))
		return profile;

	char *profileName;
	system_info_get_platform_string("http://tizen.org/feature/profile", &profileName);
	switch (*profileName) {
		case 'm':
		case 'M':
			profile = TIZEN_PROFILE_MOBILE;
			break;
		case 'w':
		case 'W':
			profile = TIZEN_PROFILE_WEARABLE;
			break;
		case 't':
		case 'T':
			profile = TIZEN_PROFILE_TV;
			break;
		case 'i':
		case 'I':
			profile = TIZEN_PROFILE_IVI;
			break;
		default: // common or unknown ==> ALL ARE COMMON.
			profile = TIZEN_PROFILE_COMMON;
	}
	free(profileName);

	return profile;
}

static Evas_Object *create_conformant(Evas_Object * parent)
{
	LS_FUNC_ENTER
	Evas_Object *conformant = NULL;

	conformant = elm_conformant_add(parent);
	elm_win_conformant_set(parent, EINA_TRUE);
	elm_win_resize_object_add(parent, conformant);
	evas_object_size_hint_weight_set(conformant, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(conformant, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(conformant);

	LS_FUNC_EXIT
	return conformant;
}

static Evas_Object *create_layout(Evas_Object * parent)
{
	LS_FUNC_ENTER
	Evas_Object *layout = NULL;

	if (parent != NULL) {
		layout = elm_layout_add(parent);
		elm_object_content_set(parent, layout);
		evas_object_show(layout);
	}

	LS_FUNC_EXIT
	return layout;
}

static void win_del(void *data, Evas_Object * obj, void *event)
{
	LS_FUNC_ENTER
	elm_exit();
}

static void save_vconf(int value)
{
	int enabled = 0;
	int ret = 0;

	ret = vconf_get_int(VCONFKEY_LOCATION_HEREMAPS_CONSENT, &enabled);
	if (ret != 0)
		LS_LOGE("Fail to get vconf value");
	else if (enabled != value) {
		ret = vconf_set_int(VCONFKEY_LOCATION_HEREMAPS_CONSENT, value);
		if (ret != 0)
			LS_LOGE("Fail to set vconf value");
	}
}

static void disagree_btn_cb(void *data, Evas_Object * obj, void *event)
{
	LS_FUNC_ENTER
	heremaps_uc_app_data *ad = (heremaps_uc_app_data *) data;
	LS_RETURN_IF_FAILED(ad);

	save_vconf(0);
	ad->btn_clicked = 1;

	elm_exit();
}

static void agree_btn_cb(void *data, Evas_Object * obj, void *event)
{
	LS_FUNC_ENTER
	heremaps_uc_app_data *ad = (heremaps_uc_app_data *) data;
	LS_RETURN_IF_FAILED(ad);

	save_vconf(1);
	ad->btn_clicked = 1;

	elm_exit();
}

static void back_btn_cb(void *data, Evas_Object * obj, void *event)
{
	LS_FUNC_ENTER
	heremaps_uc_app_data *ad = (heremaps_uc_app_data *) data;
	LS_RETURN_IF_FAILED(ad);

	save_vconf(0);
	ad->btn_clicked = 1;
	elm_exit();
}

static Evas_Object *create_win(const char *name)
{
	LS_FUNC_ENTER
	Evas_Object *eo;

	eo = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (eo) {
		elm_win_title_set(eo, name);
		elm_win_borderless_set(eo, EINA_TRUE);
		evas_object_smart_callback_add(eo, "delete,request", win_del, NULL);
		elm_win_alpha_set(eo, EINA_TRUE);
		if (elm_win_wm_rotation_supported_get(eo)) {
			int rots[4] = { 0, 90, 180, 270 };
			elm_win_wm_rotation_available_rotations_set(eo, (const int *)(&rots), 4);
		}
	}
	return eo;
}

static void anchor_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	LS_FUNC_ENTER
	Elm_Entry_Anchor_Info *ai = event_info;
	LS_RETURN_IF_FAILED(ai);

	app_control_h ac = NULL;
	app_control_create(&ac);
	app_control_set_operation(ac, APP_CONTROL_OPERATION_VIEW);
	app_control_set_uri(ac, ai->name);
	app_control_send_launch_request(ac, NULL, NULL);
	app_control_destroy(ac);
}

static void get_content(char *buf, char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vsnprintf(buf, 4096, fmt, ap);
	va_end(ap);
}

static Evas_Object *create_popup_tv(Evas_Object *layout, heremaps_uc_app_data *ad)
{
	LS_FUNC_ENTER

	/* popup */
	Evas_Object *popup = elm_popup_add(ad->win_main);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, back_btn_cb, ad);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(popup, "title,text", P_("IDS_POSITIONING_CONSENT_TITLE"));

	char buf[4096] = {0};
	char *str2 = "<color=#006fd1ff underline=on underline_color=#006fd1ff><a href=http://here.com/terms/service-terms>http://here.com/terms/service-terms</a></color>";
	char *str3 = "<color=#006fd1ff underline=on underline_color=#006fd1ff><a href=http://here.com/privacy/privacy-policy>http://here.com/privacy/privacy-policy</a></color>";
	get_content(buf, P_("IDS_POSITIONING_CONSENT_BODY"), str2, str3);
	char *text = strdup(buf);
	elm_object_text_set(popup, text);

	/* Disagree button */
	Evas_Object *disagree_btn = elm_button_add(popup);
	elm_object_style_set(disagree_btn, "popup");
	elm_object_text_set(disagree_btn, P_("IDS_ST_BUTTON_DISAGREE"));
	elm_object_part_content_set(popup, "button1", disagree_btn);
	evas_object_smart_callback_add(disagree_btn, "clicked", disagree_btn_cb, ad);

	/* Agree button */
	Evas_Object *agree_btn = elm_button_add(popup);
	elm_object_style_set(agree_btn, "popup");
	elm_object_text_set(agree_btn, P_("IDS_ST_BUTTON_AGREE"));
	elm_object_part_content_set(popup, "button2", agree_btn);
	evas_object_smart_callback_add(agree_btn, "clicked", agree_btn_cb, ad);

	evas_object_show(popup);

	LS_FUNC_EXIT
	return popup;
}

static Evas_Object *create_popup_wearable(Evas_Object *layout, heremaps_uc_app_data *ad)
{
	LS_FUNC_ENTER

	/* popup */
	Evas_Object *popup = elm_popup_add(layout);
	elm_object_style_set(popup, "circle");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_FILL, EVAS_HINT_FILL);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, back_btn_cb, ad);

	Evas_Object *popupLayout = elm_layout_add(popup);
	elm_layout_theme_set(popupLayout, "layout", "popup", "content/circle/buttons1");
	evas_object_size_hint_align_set(popupLayout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(popupLayout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(popupLayout, "elm.text.title", P_("IDS_POSITIONING_CONSENT_TITLE"));
	elm_object_content_set(popup, popupLayout);

	Evas_Object *box = elm_box_add(popupLayout);
	elm_box_align_set(box, 0.5, 0);
	elm_object_part_content_set(popupLayout, "elm.swallow.content", box);
	evas_object_show(box);

	char buf[4096] = {0};
	char *str2 = "http://here.com/terms/service-terms";
	char *str3 = "http://here.com/privacy/privacy-policy";
	get_content(buf, P_("IDS_POSITIONING_CONSENT_BODY"), str2, str3);
	char *text = strdup(buf);

	Evas_Object *entry = elm_entry_add(box);
	evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, 0.0);
	evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_entry_editable_set(entry, EINA_FALSE);
	elm_object_part_text_set(entry, "elm.text", text);
	elm_entry_text_style_user_push(entry, "DEFAULT='align=center'");
	elm_box_pack_end(box, entry);
	evas_object_show(entry);

	/* Agree button */
	Evas_Object *agree_btn = elm_button_add(popup);
	elm_object_style_set(agree_btn, "popup/circle");
	evas_object_size_hint_weight_set(agree_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(agree_btn, "elm.text", P_("IDS_ST_BUTTON_AGREE"));

	evas_object_smart_callback_add(agree_btn, "clicked", agree_btn_cb, ad);
	elm_object_part_content_set(popup, "button1", agree_btn);

	evas_object_show(popup);

	LS_FUNC_EXIT
	return popup;
}

static Evas_Object *create_popup(Evas_Object *layout, heremaps_uc_app_data *ad)
{
	LS_FUNC_ENTER

	/* popup */
	Evas_Object *popup = elm_popup_add(layout);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, back_btn_cb, ad);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_text_set(popup, "title,text", P_("IDS_POSITIONING_CONSENT_TITLE"));

	Evas_Object *box = elm_box_add(popup);
	elm_box_align_set(box, 0.5, 0);
	elm_object_part_content_set(popup, "elm.swallow.content", box);
	evas_object_show(box);

	char buf[4096] = {0};
	char *str2 = "<color=#006fd1ff underline=on underline_color=#006fd1ff><a href=http://here.com/terms/service-terms>http://here.com/terms/service-terms</a></color>";
	char *str3 = "<color=#006fd1ff underline=on underline_color=#006fd1ff><a href=http://here.com/privacy/privacy-policy>http://here.com/privacy/privacy-policy</a></color>";
	get_content(buf, P_("IDS_POSITIONING_CONSENT_BODY"), str2, str3);
	char *text = strdup(buf);

	Evas_Object *entry = elm_entry_add(box);
	evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, 0.0);
	evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_entry_editable_set(entry, EINA_FALSE);
	elm_object_part_text_set(entry, "elm.text", text);
	elm_entry_text_style_user_push(entry, "DEFAULT='align=center'");
	evas_object_smart_callback_add(entry, "anchor,clicked", anchor_clicked_cb, NULL);
	elm_box_pack_end(box, entry);
	evas_object_show(entry);

	/* Disagree button */
	Evas_Object *disagree_btn = elm_button_add(popup);
	elm_object_style_set(disagree_btn, "popup");
	elm_object_text_set(disagree_btn, P_("IDS_ST_BUTTON_DISAGREE"));
	elm_object_part_content_set(popup, "button1", disagree_btn);
	evas_object_smart_callback_add(disagree_btn, "clicked", disagree_btn_cb, ad);

	/* Agree button */
	Evas_Object *agree_btn = elm_button_add(popup);
	elm_object_style_set(agree_btn, "popup");
	elm_object_text_set(agree_btn, P_("IDS_ST_BUTTON_AGREE"));
	elm_object_part_content_set(popup, "button2", agree_btn);
	evas_object_smart_callback_add(agree_btn, "clicked", agree_btn_cb, ad);

	evas_object_show(popup);

	LS_FUNC_EXIT
	return popup;
}

static bool _app_create_cb(void *user_data)
{
	LS_FUNC_ENTER

	return true;
}

static void _app_terminate_cb(void *user_data)
{
	LS_FUNC_ENTER
}

static void _app_pause_cb(void *user_data)
{
	LS_FUNC_ENTER
	heremaps_uc_app_data *ad = (heremaps_uc_app_data *) user_data;
	LS_RETURN_IF_FAILED(ad);

	if (ad->btn_clicked == 0)
		save_vconf(0);

	elm_exit();
}

static void _app_resume_cb(void *user_data)
{
	LS_FUNC_ENTER
}

static void _app_control_cb(app_control_h app_control, void *user_data)
{
	LS_FUNC_ENTER

	heremaps_uc_app_data *ad = (heremaps_uc_app_data *) user_data;
	LS_RETURN_IF_FAILED(ad);

	if (ad->win_main) {
		evas_object_del(ad->win_main);
		ad->win_main = NULL;
	}

	elm_app_base_scale_set(2.6);
	elm_config_accel_preference_set("3d");

	bindtextdomain(HEREMAPS_UC_PKG, LOCALE_DIR);

	save_vconf(-1);
	ad->btn_clicked = 0;

	ad->win_main = create_win(HEREMAPS_UC_PKG);
	ad->conformant = create_conformant(ad->win_main);
	ad->layout_main = create_layout(ad->conformant);

	if (_get_tizen_profile() == TIZEN_PROFILE_WEARABLE)
		ad->popup = create_popup_wearable(ad->layout_main, ad);
	else if (_get_tizen_profile() == TIZEN_PROFILE_TV)
		ad->popup = create_popup_tv(ad->layout_main, ad);
	else
		ad->popup = create_popup(ad->layout_main, ad);

	evas_object_show(ad->win_main);
	LS_FUNC_EXIT
}

/*
static void _app_low_memory_cb(void *user_data)
{
	LS_FUNC_ENTER
}

static void _app_low_battery_cb(void *user_data)
{
	LS_FUNC_ENTER
}

static void _app_device_orientation_cb(app_event_info_h event_info, void *user_data)
{
	LS_FUNC_ENTER
	LS_RETURN_IF_FAILED(event_info);
	LS_RETURN_IF_FAILED(user_data);

	heremaps_uc_app_data *ad = (heremaps_uc_app_data *)user_data;
	app_device_orientation_e orientation;
	app_event_get_device_orientation(event_info, &orientation);
	elm_win_rotation_with_resize_set(ad->win_main, orientation);
}
*/

static void _app_language_changed_cb(app_event_info_h event_info, void *user_data)
{
	LS_FUNC_ENTER

	char *locale = vconf_get_str(VCONFKEY_LANGSET);
	if (locale) {
		elm_language_set(locale);
		free(locale);
	}
}

int main(int argc, char *argv[])
{
	LS_FUNC_ENTER

	int ret = 0;
	heremaps_uc_app_data ad = {0,};

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = _app_create_cb;
	event_callback.terminate = _app_terminate_cb;
	event_callback.app_control = _app_control_cb;
	event_callback.pause = _app_pause_cb;
	event_callback.resume = _app_resume_cb;

	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, _app_language_changed_cb, NULL);

	ret = APP_ERROR_NONE;
	ret = ui_app_main(argc, argv, &event_callback, &ad);

	if (ret != APP_ERROR_NONE)
		LS_LOGE("ui_app_main() is failed. err=%d", ret);

	return ret;

	LS_FUNC_EXIT
}
